from NWS_Weather import current_weather, predicted_weather

for weather in current_weather(zipcode="53818"):
    print(weather)


for weather in predicted_weather(gridX=91, gridY=13, gridId='ARX'):
    print(weather)


import requests
from typing import List

from geopy.geocoders import Nominatim

from .observation import Observation

def current_weather(**kwargs) -> List[Observation]:
    """Get the current weather for a given location

    Raises:
        Exception: Could not get the current weather for the given location

    Returns:
        List[Observation]: A list of observations for the given location
    """
    station = ""

    if 'station' in kwargs:
        station = kwargs['station']
        
    elif 'lat' in kwargs and 'lon' in kwargs:
        lat = kwargs['lat']
        lon = kwargs['lon']
        station = get_station(lat, lon)
        
    elif 'zipcode' in kwargs:
        station = get_station_zipcode(kwargs['zipcode'])
        
    r = requests.get(f"https://api.weather.gov/stations/{station}/observations")
    
    if r.status_code != 200:
        print(r)
        raise Exception(f"Error getting current weather")
    
    observations = []
    
    for d in r.json()['features']:
        observations.append(Observation(d))
        
    return observations

def get_station(lat: float, lon: float) -> str:
    """Find the nearest station to a lat/lon pair

    Args:
        lat (float): Latitude that you want to find the nearest station for
        lon (float): Longitude that you want to find the nearest station for

    Raises:
        Exception: Could not find a station for the lat/lon pair

    Returns:
        str: The nearest station to the lat/lon pair
    """
    
    url = f"https://api.weather.gov/points/{round(lat, 4)},{round(lon, 4)}"
    r = requests.get(url)
    
    if r.status_code == 200:
        observationStationsURL = r.json()['properties']['observationStations']
        observationStationR = requests.get(observationStationsURL)
        
        if observationStationR.status_code == 200:
            return observationStationR.json()['features'][0]['properties']['stationIdentifier']
    
    print(r)
    print(observationStationR)
    raise Exception(f"Error getting station for lat/lon: {lat}, {lon}")
    

def get_station_zipcode(zipcode: str) -> str:
    """Get's the nearest station to a zipcode

    Args:
        zipcode (str): zipcode that you want to find the nearest station for

    Returns:
        str: The nearest station to the zipcode
    """
    
    geolocator = Nominatim(user_agent="zipcode_converter")
    location = geolocator.geocode(zipcode)
    
    return get_station(location.latitude, location.longitude)


import requests
from typing import List

from geopy.geocoders import Nominatim

from .forcast import forcast

def predicted_weather(**kwargs) -> List[forcast]:

    grid = ("", 0, 0)

    if 'gridX' in kwargs and 'gridY' in kwargs and 'gridId' in kwargs:
        grid = (
            kwargs['gridId'],
            kwargs['gridX'],
            kwargs['gridY'],
        )
        
    elif 'lat' in kwargs and 'lon' in kwargs:
        lat = kwargs['lat']
        lon = kwargs['lon']
        grid = get_forecast_grid(lat, lon)
        
    elif 'zipcode' in kwargs:
        grid = get_forecast_grid_zipcode(kwargs['zipcode'])
        
    r = requests.get(f"https://api.weather.gov/gridpoints/{grid[0]}/{grid[1]},{grid[2]}/forecast/hourly")
    
    if r.status_code != 200:
        print(r)
        raise Exception(f"Error getting predicted weather")
    
    forcasts = []
    
    for d in r.json()["properties"]["periods"]:
        forcasts.append(forcast(d))
        
    return forcasts    

def get_forecast_grid(lat: int, lon: int) -> (str, int, int):

    url = f"https://api.weather.gov/points/{round(lat, 4)},{round(lon, 4)}"
    r = requests.get(url)
    
    if r.status_code == 200:        
        return (
            r.json()['properties']['gridId'],
            r.json()['properties']['gridX'],
            r.json()['properties']['gridY']
        )

    print(r)
    raise Exception(f"Error getting forecast grid")
    

def get_forecast_grid_zipcode(zipcode: str) ->  (str, int, int):
    
    geolocator = Nominatim(user_agent="zipcode_converter")
    location = geolocator.geocode(zipcode)
    print(location.latitude, location.longitude)
    
    return get_forecast_grid(location.latitude, location.longitude)


from datetime import datetime

class PresentWeather:
    data: str
    
    def __init__(self, data):
        self.data = data

    def __str__(self):
        return ", ".join(self.data)

class CloudLayers:
    layers: list
    
    def __init__(self, layers: list):
        self.layers = layers

    def __str__(self):
        s = "Cloud Layers:"
        
        for i, layer in enumerate(self.layers):
            s += f"""
Layer {i + 1}:
    Amount: {layer['amount']}
    Base: {layer['base']['value']} m
"""
            
        return s

class Observation:
    timestamp: datetime
    
    rawMessage: str
    textDescription: str
    icon: str
    presentWeather: PresentWeather
    
    temperature: float
    dewpoint: float
    windDirection: str
    windSpeed: float
    windGust: float
    barometricPressure: float
    seaLevelPressure: float
    visibility: float
    maxTemperatureLast24Hours: float
    minTemperatureLast24Hours: float
    precipitationLastHour: float
    precipitationLast3Hours: float
    precipitationLast6Hours: float
    relativeHumidity: float
    windChill: float
    heatIndex: float
    
    cloudLayers: CloudLayers
    
    def __init__(self, data: dict):
        """ Initizes the observation class

        Args:
            data (dict): Data from the NWS API
        """
        
        self.timestamp = datetime.strptime(data['properties']['timestamp'], "%Y-%m-%dT%H:%M:%S%z")
        
        self.rawMessage = data['properties']['rawMessage']
        self.textDescription = data['properties']['textDescription']
        self.icon = data['properties']['icon']
        self.presentWeather = PresentWeather(data['properties']['presentWeather'])
        
        self.temperature = data['properties']['temperature']['value']
        self.dewpoint = data['properties']['dewpoint']['value']
        self.windDirection = data['properties']['windDirection']['value']
        self.windSpeed = data['properties']['windSpeed']['value']
        self.windGust = data['properties']['windGust']['value']
        self.barometricPressure = data['properties']['barometricPressure']['value']
        self.seaLevelPressure = data['properties']['seaLevelPressure']['value']
        self.visibility = data['properties']['visibility']['value']
        self.maxTemperatureLast24Hours = data['properties']['maxTemperatureLast24Hours']['value']
        self.minTemperatureLast24Hours = data['properties']['minTemperatureLast24Hours']['value']
        self.precipitationLastHour = data['properties']['precipitationLastHour']['value']
        self.precipitationLast3Hours = data['properties']['precipitationLast3Hours']['value']
        self.precipitationLast6Hours = data['properties']['precipitationLast6Hours']['value']
        self.relativeHumidity = data['properties']['relativeHumidity']['value']
        self.windChill = data['properties']['windChill']['value']
        self.heatIndex = data['properties']['heatIndex']['value']
        
        self.cloudLayers = CloudLayers(data['properties']['cloudLayers'])
        
    def __str__(self):
        return f"""{self.timestamp}

{self.rawMessage}
{self.textDescription}
{self.icon}
{self.presentWeather}

Temperature: {self.temperature} C
Dewpoint: {self.dewpoint} C
Wind Direction: {self.windDirection} Deg
Wind Speed: {self.windSpeed} KPH
Wind Gust: {self.windGust} KPH
Barometric Pressure: {self.barometricPressure} pa
Sea Level Pressure: {self.seaLevelPressure} pa
Visibility: {self.visibility} m
Max Temperature Last 24 Hours: {self.maxTemperatureLast24Hours} C
Min Temperature Last 24 Hours: {self.minTemperatureLast24Hours} C
Precipitation Last Hour: {self.precipitationLastHour} mm
Precipitation Last 3 Hours: {self.precipitationLast3Hours} mm
Precipitation Last 6 Hours: {self.precipitationLast6Hours} mm
Relative Humidity: {self.relativeHumidity} %
Wind Chill: {self.windChill} C
Heat Index: {self.heatIndex} C

{self.cloudLayers}"""


from datetime import datetime

class forcast:
    startTime: datetime
    endTime: datetime
    
    isDaytime: bool
    temperature: float
    temperatureTrend: str
    probabilityOfPrecipitation: int
    dewPoint: float
    windSpeed: int
    windDirection: str
    
    icon: str
    shortForecast: str
    detailedForecast: str
    
    def __init__(self, data: dict):
        """ Initizes the forcast class

        Args:
            data (dict): Data from the NWS API
        """
        
        self.startTime = datetime.strptime(data['startTime'], "%Y-%m-%dT%H:%M:%S%z")
        self.endTime = datetime.strptime(data['endTime'], "%Y-%m-%dT%H:%M:%S%z")
        
        self.isDaytime = data['isDaytime']
        self.temperature = data['temperature']
        self.temperatureTrend = data['temperatureTrend']
        self.probabilityOfPrecipitation = data['probabilityOfPrecipitation']["value"]
        self.dewPoint = data['dewpoint']["value"]
        self.windSpeed = int(data['windSpeed'].split(" ")[0])
        self.windDirection = data['windDirection']
        
        self.icon = data['icon']
        self.shortForecast = data['shortForecast']
        self.detailedForecast = data['detailedForecast']
        
        
    def __str__(self):
        return f"""{self.startTime} - {self.endTime}
{self.shortForecast}
{self.detailedForecast}
Temperature: {self.temperature}°F
Temperature Trend: {self.temperatureTrend}
Probability of Precipitation: {self.probabilityOfPrecipitation}%
Dew Point: {self.dewPoint}°F
Wind Speed: {self.windSpeed} mph
Wind Direction: {self.windDirection}
"""


from setuptools import setup, find_packages

setup(
    name='NWS_Weather',
    version='1',
    packages=find_packages(),
    install_requires=[
        'requests',
        'geopy'
    ],
    author='Colin Sutter',
    author_email='contact@csutter.dev',
    description='A package for retrieving weather data from the National Weather Service',
    url='https://github.com/Pizzarules668/NWS_Weather',
)
